@@ -12,7 +12,6 @@ from torch import nn, optim
 from torch.utils.data import DataLoader
 
 from .. import logger, task_logger
-from ..contractor.task_contractor import AutoTaskContractor
 from ..fed_avg import FedAvgScheduler
 from ..fed_avg.contractor import AutoFedAvgContractor
 from .exceptions import AutoModelError, ConfigError
@@ -103,8 +102,8 @@ class MandatoryConfig:
 class Preprocessor(ABC):
 
     @abstractmethod
-    def transform(self, image_file: str) -> torch.Tensor:
-        """Transform an image object into an input tensor."""
+    def transform(self, *args: Any, **kwargs: Any) -> torch.Tensor:
+        """Transform a raw input object into an input tensor."""
 
     def __call__(self, *args: Any, **kwargs: Any) -> Any:
         return self.transform(*args, **kwargs)
@@ -208,7 +207,6 @@ class AutoFedAvgModel(AutoModel):
     def optimizer(self) -> optim.Optimizer:
         """Return the optimizer object used in training."""
 
-    @abstractmethod
     def state_dict(self) -> Dict[str, torch.Tensor]:
         """Get the params that need to train and update.
 
@@ -217,13 +215,14 @@ class AutoFedAvgModel(AutoModel):
         Return:
             List[torch.Tensor], The list of model params.
         """
+        return self.model.state_dict()
 
-    @abstractmethod
     def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
         """Load the params that trained and updated.
 
         Only the params returned by state_dict() should be loaded by this function.
         """
+        self.model.load_state_dict(state_dict)
 
     @abstractmethod
     def train_an_epoch(self) -> Any:
@@ -237,8 +236,6 @@ class AutoFedAvgModel(AutoModel):
         """Run a round of validation."""
         raise NotImplementedError()
 
-    @property
-    @abstractmethod
     def fine_tuned_files_dict(self) -> Optional[Dict[str, str]]:
         """Return the fine-tuned files used by reinitializing the fine-tuned model.
 
@@ -254,6 +251,7 @@ class AutoFedAvgModel(AutoModel):
             'relative_path_to_resource_dir_root': 'real_path_to_access_file'
         }
         """
+        return {}
 
     @property
     def result_dir(self):
@@ -270,8 +268,6 @@ class AutoFedAvgModel(AutoModel):
                         scheduler_impl: Type['AutoFedAvgScheduler'],
                         is_initiator: bool = False,
                         recover: bool = False,
-                        min_clients: int = 0,
-                        max_clients: int = 0,
                         max_rounds: int = 0,
                         merge_epochs: int = 1,
                         calculation_timeout: int = 300,
@@ -288,20 +284,10 @@ class AutoFedAvgModel(AutoModel):
         if not is_initiator and not self.training_loader:
             raise AutoModelError('The collaborator must provide training dataset.')
 
-        task_contractor = AutoTaskContractor(task_id=task_id)
-        self.clients = task_contractor.query_nodes()
         involve_aggregator = bool(self.training_loader
                                   and len(self.training_loader) > 0)
-        if involve_aggregator:
-            min_clients = min_clients or len(self.clients)
-            max_clients = max_clients or len(self.clients)
-        else:
-            min_clients = min_clients or len(self.clients) - 1
-            max_clients = max_clients or len(self.clients) - 1
 
         self.scheduler = scheduler_impl(auto_proxy=self,
-                                        min_clients=min_clients,
-                                        max_clients=max_clients,
                                         max_rounds=max_rounds,
                                         merge_epochs=merge_epochs,
                                         calculation_timeout=calculation_timeout,
@@ -322,8 +308,6 @@ class AutoFedAvgScheduler(FedAvgScheduler):
 
     def __init__(self,
                  auto_proxy: AutoFedAvgModel,
-                 min_clients: int,
-                 max_clients: int,
                  max_rounds: int = 0,
                  merge_epochs: int = 1,
                  calculation_timeout: int = 300,
@@ -331,9 +315,7 @@ class AutoFedAvgScheduler(FedAvgScheduler):
                  log_rounds: int = 0,
                  involve_aggregator: bool = False,
                  **kwargs):
-        super().__init__(min_clients=min_clients,
-                         max_clients=max_clients,
-                         max_rounds=max_rounds,
+        super().__init__(max_rounds=max_rounds,
                          merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
@@ -363,9 +345,9 @@ class AutoFedAvgScheduler(FedAvgScheduler):
         return self.auto_proxy.load_state_dict(state_dict)
 
     @property
-    @abstractmethod
     def best_state_dict(self) -> Dict[str, torch.Tensor]:
         """Return the best state of the model by now."""
+        return self.state_dict()
 
     def _init_contractor(self):
         return AutoFedAvgContractor(task_id=self.task_id)
@@ -396,16 +378,8 @@ class AutoFedAvgScheduler(FedAvgScheduler):
         """
         self.push_log('Uploading task achievement and closing task ...')
 
-        # TODO remove metrics_files later
-        metrics_files = []
-        for _name, _metrics in self._metrics_bucket.items():
-            _file = f'{os.path.join(self._result_dir, _name)}.csv'
-            _metrics.to_csv(_file)
-            metrics_files.append(_file)
         report_file = os.path.join(self._result_dir, "report.zip")
         with ZipFile(report_file, 'w') as report_zip:
-            for _file in metrics_files:
-                report_zip.write(_file, os.path.basename(_file))
             for path, _, filenames in os.walk(self._log_dir):
                 rel_dir = os.path.relpath(path=path, start=self._result_dir)
                 rel_dir = rel_dir.lstrip('.')  # ./file => file
@@ -419,7 +393,7 @@ class AutoFedAvgScheduler(FedAvgScheduler):
         resource_zip_file = os.path.join(self._result_dir, 'model.zip')
 
         with ZipFile(resource_zip_file, 'w') as resource_zip:
-            fine_tuned_files_dict = self.auto_proxy.fine_tuned_files_dict or {}
+            fine_tuned_files_dict = self.auto_proxy.fine_tuned_files_dict() or {}
             # package files of pretrained model
             for path, _, filenames in os.walk(resource_dir):
                 rel_dir = os.path.relpath(path=path, start=resource_dir)
