@@ -336,19 +336,14 @@ class _LabelSmoothingCrossEntropy(object):
 class DemoFedIRM(FedAvgScheduler):
 
     def __init__(self,
-                 min_clients: int,
-                 max_clients: int,
                  root_path: str,
                  csv_file_train: str,
                  csv_file_test: str,
-                 name: str = None,
                  max_rounds: int = 0,
                  merge_epochs: int = 1,
                  calculation_timeout: int = 300,
-                 perf_bench_timeout: int = 30,
                  schedule_timeout: int = 30,
                  log_rounds: int = 0,
-                 is_centralized: bool = True,
                  is_deterministic: bool = True,
                  seed: int = 1337,
                  label_uncertainty: str = 'U-Ones',
@@ -356,16 +351,11 @@ class DemoFedIRM(FedAvgScheduler):
                  batch_size: int = 16,
                  base_lr: float = 2e-4,
                  involve_aggregator: bool = False):
-        super().__init__(min_clients=min_clients,
-                         max_clients=max_clients,
-                         name=name,
-                         max_rounds=max_rounds,
+        super().__init__(max_rounds=max_rounds,
                          merge_epochs=merge_epochs,
                          calculation_timeout=calculation_timeout,
-                         perf_bench_timeout=perf_bench_timeout,
                          schedule_timeout=schedule_timeout,
                          log_rounds=log_rounds,
-                         is_centralized=is_centralized,
                          involve_aggregator=involve_aggregator)
         self.is_deterministic = is_deterministic
         self.seed = seed
@@ -573,7 +563,7 @@ class DemoFedIRM(FedAvgScheduler):
 
         return AUROCs, Accus, Senss, Specs, pre, F1
 
-    def test(self):
+    def run_test(self):
         normalize = transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
         test_dataset = _CheXpertDataset(
             root_dir=self.root_path,
