@@ -36,7 +36,7 @@ class ConvNet(nn.Module):
         self.conv2 = nn.Conv2d(in_channels=10, out_channels=20, kernel_size=5)
         self.conv2_drop = nn.Dropout2d()
         self.fc1 = nn.Linear(in_features=80, out_features=50)
-        self.fc2 = nn.Linear(in_features=50, out_features=20)
+        self.fc2 = nn.Linear(in_features=50, out_features=10)
 
     def forward(self, x):
         x = F.relu(F.max_pool2d(self.conv1(x), 2))
@@ -51,7 +51,7 @@ class InferModule(nn.Module):
 
     def __init__(self) -> None:
         super().__init__()
-        self.fc1 = nn.Linear(40, 20)
+        self.fc1 = nn.Linear(20, 20)
         self.fc2 = nn.Linear(20, 10)
 
     def forward(self, input):
@@ -66,13 +66,11 @@ class DemoHeteroHost(HeteroNNHostScheduler):
                  feature_key: str,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
                  log_rounds: int = 0) -> None:
         super().__init__(feature_key=feature_key,
-                         name=name,
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
@@ -96,10 +94,10 @@ class DemoHeteroHost(HeteroNNHostScheduler):
 
         return set(train_ids), set(test_ids)
 
-    def make_feature_model(self) -> nn.Module:
+    def build_feature_model(self) -> nn.Module:
         return ConvNet()
 
-    def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
+    def build_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
         return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
 
     def _erase_right(self, _image: torch.Tensor) -> torch.Tensor:
@@ -151,10 +149,10 @@ class DemoHeteroHost(HeteroNNHostScheduler):
             _data = self._erase_right(_data)
             yield feature_model(_data), _labels
 
-    def make_infer_model(self) -> nn.Module:
+    def build_infer_model(self) -> nn.Module:
         return InferModule()
 
-    def make_infer_optimizer(self, infer_model: nn.Module) -> optim.Optimizer:
+    def build_infer_optimizer(self, infer_model: nn.Module) -> optim.Optimizer:
         return optim.SGD(infer_model.parameters(), lr=0.01, momentum=0.9)
 
     def train_a_batch(self, feature_projection: Dict[str, torch.Tensor], labels: torch.Tensor):
@@ -166,9 +164,9 @@ class DemoHeteroHost(HeteroNNHostScheduler):
         loss.backward()
         self.optimizer.step()
 
-    def test(self,
-             batched_feature_projections: List[torch.Tensor],
-             batched_labels: List[torch.Tensor]):
+    def run_test(self,
+                 batched_feature_projections: List[torch.Tensor],
+                 batched_labels: List[torch.Tensor]):
         start = time()
         test_loss = 0
         correct = 0
@@ -202,11 +200,9 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
                  feature_key: str,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  schedule_timeout: int = 30,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
-                         name=name,
                          schedule_timeout=schedule_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
@@ -228,10 +224,10 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
 
         return set(train_ids), set(test_ids)
 
-    def make_feature_model(self) -> nn.Module:
+    def build_feature_model(self) -> nn.Module:
         return ConvNet()
 
-    def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
+    def build_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
         return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
 
     def _erase_left(self, _image: torch.Tensor) -> torch.Tensor:
@@ -292,7 +288,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
                  project_layer_lr: float,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
@@ -301,7 +296,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         super().__init__(feature_key=feature_key,
                          project_layer_config=project_layer_config,
                          project_layer_lr=project_layer_lr,
-                         name=name,
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
@@ -326,10 +320,10 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
 
         return set(train_ids), set(test_ids)
 
-    def make_feature_model(self) -> nn.Module:
+    def build_feature_model(self) -> nn.Module:
         return ConvNet()
 
-    def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
+    def build_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
         return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
 
     def _erase_right(self, _image: torch.Tensor) -> torch.Tensor:
@@ -382,10 +376,10 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
             _data = self._erase_right(_data)
             yield feature_model(_data), _labels
 
-    def make_infer_model(self) -> nn.Module:
+    def build_infer_model(self) -> nn.Module:
         return InferModule()
 
-    def make_infer_optimizer(self, infer_model: nn.Module) -> optim.Optimizer:
+    def build_infer_optimizer(self, infer_model: nn.Module) -> optim.Optimizer:
         return optim.SGD(infer_model.parameters(), lr=0.01, momentum=0.9)
 
     def train_a_batch(self, feature_projection: Dict[str, torch.Tensor], labels: torch.Tensor):
@@ -397,9 +391,9 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         loss.backward()
         self.optimizer.step()
 
-    def test(self,
-             batched_feature_projections: List[torch.Tensor],
-             batched_labels: List[torch.Tensor]):
+    def run_test(self,
+                 batched_feature_projections: List[torch.Tensor],
+                 batched_labels: List[torch.Tensor]):
         start = time()
         test_loss = 0
         correct = 0
@@ -434,12 +428,10 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
                  project_layer_lr: int,
                  batch_size: int,
                  data_dir: str,
-                 name: str = None,
                  schedule_timeout: int = 30,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
                          project_layer_lr=project_layer_lr,
-                         name=name,
                          schedule_timeout=schedule_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
@@ -461,10 +453,10 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
 
         return set(train_ids), set(test_ids)
 
-    def make_feature_model(self) -> nn.Module:
+    def build_feature_model(self) -> nn.Module:
         return ConvNet()
 
-    def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
+    def build_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
         return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
 
     def _erase_left(self, _image: torch.Tensor) -> torch.Tensor:
@@ -534,14 +526,13 @@ def get_host(mode: str = VANILLA) -> Union[DemoHeteroHost, DemoSecureHeteroHost]
         scheduler = DemoHeteroHost(feature_key='demo_host',
                                    batch_size=128,
                                    data_dir=_DATA_DIR,
-                                   name='demo_hetero_host',
                                    max_rounds=5,
                                    calculation_timeout=60,
                                    log_rounds=1)
     else:
         project_layer_config = [
-            ('demo_host', 20, 20),
-            ('demo_collaborator', 20, 20)
+            ('demo_host', 10, 10),
+            ('demo_collaborator', 10, 10)
         ]
         # Too big batch size could kill the server
         scheduler = DemoSecureHeteroHost(feature_key='demo_host',
@@ -549,8 +540,7 @@ def get_host(mode: str = VANILLA) -> Union[DemoHeteroHost, DemoSecureHeteroHost]
                                          project_layer_lr=0.01,
                                          batch_size=128,
                                          data_dir=_DATA_DIR,
-                                         name='demo_hetero_host',
-                                         max_rounds=5,
+                                         max_rounds=10,
                                          calculation_timeout=60,
                                          log_rounds=1)
 
@@ -574,16 +564,14 @@ def get_collaborator(mode: str = VANILLA) -> Union[DemoHeteroCollaborator,
     if mode == VANILLA:
         scheduler = DemoHeteroCollaborator(feature_key='demo_collaborator',
                                            batch_size=128,
-                                           data_dir=_DATA_DIR,
-                                           name='demo_hetero_collaborator')
+                                           data_dir=_DATA_DIR)
 
     else:
         # Too big batch size could kill the server
         scheduler = DemoSecureHeteroCollaborator(feature_key='demo_collaborator',
                                                  project_layer_lr=0.01,
                                                  batch_size=128,
-                                                 data_dir=_DATA_DIR,
-                                                 name='demo_hetero_collaborator')
+                                                 data_dir=_DATA_DIR)
 
     with open(pickle_file, 'w+b') as pf:
         pickle.dump(scheduler, pf)
