@@ -105,14 +105,14 @@ class SimplePerfBench:
             (net_time, calc_time)
         """
         results = []
-        executor = ThreadPoolExecutor(max_workers=1)
-        executor.submit(self._run_benchmark, results)
-        for _ in range(self.TIMEOUT):
-            sleep(0.1)
-            if results and len(results) == 2:
-                net_time, calc_time = results
-                return net_time, calc_time
-        return self.TIMEOUT, self.TIMEOUT
+        with ThreadPoolExecutor(max_workers=1) as executor:
+            executor.submit(self._run_benchmark, results)
+            for _ in range(self.TIMEOUT):
+                sleep(0.1)
+                if results and len(results) == 2:
+                    net_time, calc_time = results
+                    return net_time, calc_time
+            return self.TIMEOUT, self.TIMEOUT
 
 
 @dataclass
