@@ -9,7 +9,7 @@ from zipfile import ZipFile
 
 import cloudpickle as pickle
 
-from . import logger, task_logger
+from . import is_mock_mode, logger, task_logger
 from .contractor import TaskContractor
 
 
@@ -138,10 +138,9 @@ class Scheduler(ABC):
     def push_log(self, message: str):
         """Push a running log message to the task manager."""
         assert message and isinstance(message, str), f'invalid log message: {message}'
-        if hasattr(self, 'task_id') and self.task_id:
+        if not is_mock_mode() and hasattr(self, 'task_id') and self.task_id:
             task_logger.info(message, extra={"task_id": self.task_id})
-        else:
-            logger.info(message)
+        logger.info(message)
 
     def _switch_status(self, _status: str):
         """Switch to a new status and leave a log."""
